@@ -2,15 +2,20 @@ import os
 import json
 from typing import Optional
 from bgetlib.models import DownloadProgress, QualityOptions
-import bgetlib.codec as codec
-import bgetlib.utils as utils
 
 from .runtime import Runtime
 from .utils import auto_format, ensure_file_directory_created
 
+QUALITY = QualityOptions(
+    h265=True,
+    hdr=True,
+    dolby_audio=True,
+    dolby_vision=False,
+    qhd_8k=True
+)
+
 
 def show_progress(rt: Runtime, progress: DownloadProgress):
-    progress.bar_format = ("[", 15, "]")
     progress_text = "{bar} {finished_mb:.2f}/{total_mb:.2f}M {speed_kbps:.2f}K/s avg={avg_speed_kbps:.2f}K/s".format(
         bar=f"{progress.tag}: {progress.percent:<7}{progress.bar}",
         finished_mb=progress.finished / 1024 / 1024,
@@ -18,12 +23,12 @@ def show_progress(rt: Runtime, progress: DownloadProgress):
         speed_kbps=progress.speed / 1024,
         avg_speed_kbps=progress.average_speed / 1024,
     )
-    rt.log_progress(progress_text + (" " * 10))
+    rt.log_progress(progress_text + (" " * 12))
     if progress.done:
         rt.log("")
 
 
-def downloader(name: str, extension: str = ""):
+def downloader(name: str, extension: str):
     def decorator(func):
         def wrapper(rt: Runtime, video: dict, part: Optional[int] = None):
             rt.log_tags.append(name)
@@ -43,41 +48,16 @@ def downloader(name: str, extension: str = ""):
     return decorator
 
 
-def get_av_stream_url(rt: Runtime, filename: str, aid: int, cid: int):
-    stream = rt.bapi.get_stream_url(aid, cid, QualityOptions(
-        h265=True,
-        hdr=True,
-        dolby_audio=True,
-        dolby_vision=False,
-        qhd_8k=True
-    ))
-    if rt.config.host is not None:
-        stream["audio"] = utils.replace_host(stream["audio"], rt.config.host)
-        stream["video"] = utils.replace_host(stream["video"], rt.config.host)
-    return stream
-
-
-@downloader("audio")
+@downloader("audio", extension="acc")
 def download_audio(rt: Runtime, filename: str, video: dict, part: int):
-    stream = get_av_stream_url(rt, filename, video["aid"], video["pages"][part]["cid"])
-    audio_stream = rt.bapi.get_stream(stream["audio"], "audio", rt.config.chunk_size,
-                                      callback=lambda p: show_progress(rt, p))
-    extension = "aac"
-    if stream["quality"].dolby_audio:
-        extension = "ac3"
-    if stream["quality"].flac_audio:
-        extension = "flac"
-    return codec.extract_audio(audio_stream, filename + extension, stream["quality"])
+    rt.bapi.save_stream(video["aid"], video["pages"][part]["cid"], QUALITY, filename, audio_only=True,
+                        host=rt.config.host, chunk_size=rt.config.chunk_size, callback=lambda p: show_progress(rt, p))
 
 
 @downloader("video", extension="mp4")
 def download_video(rt: Runtime, filename: str, video: dict, part: int):
-    stream = get_av_stream_url(rt, filename, video["aid"], video["pages"][part]["cid"])
-    audio_stream = rt.bapi.get_stream(stream["audio"], "audio", rt.config.chunk_size,
-                                      callback=lambda p: show_progress(rt, p))
-    video_stream = rt.bapi.get_stream(stream["video"], "video", rt.config.chunk_size,
-                                      callback=lambda p: show_progress(rt, p))
-    return codec.merge(audio_stream, video_stream, filename)
+    rt.bapi.save_stream(video["aid"], video["pages"][part]["cid"], QUALITY, filename, audio_only=False,
+                        host=rt.config.host, chunk_size=rt.config.chunk_size, callback=lambda p: show_progress(rt, p))
 
 
 @downloader("danmaku", extension="xml")
@@ -87,12 +67,10 @@ def download_danmaku(rt: Runtime, filename: str, video: dict, part: int):
         f.write(content)
 
 
-@downloader("cover")
+@downloader("cover", extension="")
 def download_cover(rt: Runtime, filename: str, video: dict, _: Optional[int]):
     basename, image = rt.bapi.get_cover_picture(video["aid"])
     extension = os.path.splitext(basename)[1]
-    if extension.startswith("."):
-        extension = extension[1:]
     filename = filename + extension
     with open(filename, "wb+") as f:
         f.write(image)
